#define _WIN32_DCOM
#include <windows.h>
#include <malloc.h>
#include <stdio.h>
#include "ComMallocSpyImpl.h"

class KCheckLeaks
{
    KComMallocSpyImpl* m_pSpy;

public:
    KCheckLeaks()
        :
        m_pSpy(0) {};

    ~KCheckLeaks()
        {
            if (m_pSpy)
            {
                HRESULT hr = CoRevokeMallocSpy();
                printf("CoRevokeMallocSpy() returns 0x%08X\n", hr);

                m_pSpy->CheckLeaks();
                m_pSpy->Release();
            }
        };   

    void Register()
        { 
            m_pSpy = new KComMallocSpyImpl( KComMallocSpyImpl::ENABLE_TRACE ); 
            HRESULT hr = CoRegisterMallocSpy(m_pSpy);
            printf("CoRegisterMallocSpy() returns 0x%08X\n", hr);
        };
};

KCheckLeaks g_CheckLeaks;

int main( int argc, char** argv )
{
    if (argc != 2)
    {
        printf("Usage: MallocSpyTest <coclass>\n");
        return 1;
    };
    
    HRESULT hr;
    hr = CoInitialize(NULL);
    printf("CoInitialize() returns 0x%08X\n", hr);

    g_CheckLeaks.Register();

    // find CLSID from coclass
    
    int Size = MultiByteToWideChar(CP_ACP, 0, argv[1], -1, NULL, 0);
    LPWSTR CoClassNameW = (LPWSTR)_alloca( 2*Size );
    MultiByteToWideChar(CP_ACP, 0, argv[1], -1, CoClassNameW, Size);

    CLSID CoClass;
	hr = CLSIDFromProgID(CoClassNameW, &CoClass);
    printf("CLSIDFromProgID() returns 0x%08X\n", hr);

    if (FAILED(hr))
    {
        printf("CoClass '%s' not found\n", argv[1]);
    }
    else
    {
        IUnknown* pUnk;
        hr = CoCreateInstance(CoClass, NULL, CLSCTX_ALL, IID_IUnknown, (void**)&pUnk);
        printf("CoCreateInstance() returns 0x%08X\n", hr);

        if (FAILED(hr))
        {
            printf("Cannot create object instance\n");
        }
        else
        {
            pUnk->Release();
            printf("Object released\n");
        }
    }

    CoUninitialize();
    printf("Called CoUninitialize()\n");

	return 0;
};